/* gui_login.c Functions concerning the login window in LyXKom. */
/* version 0.6 */

#include <string.h>
#include <suit.h>
#include <lyxkom.h>
#include <lyx_login.h>
#include <gui_protos.h>
#include <lyx_protos.h>

/* The objects of the login window */
SUIT_object loginBB, loginservinm, loginnamninm, loginpswinm, loginhelp, loginokbutton;

/* The objects of the doublecheck window */
SUIT_object dbcheckBB, dbuname, dbpsw, dbhelp, dbavbrytbutton, dbokbutton;

char *firstpsw, *secondpsw;

extern SUIT_object mainBB;

/* Callback for all buttons in the login window except the help button */

void GUI_logindo(SUIT_object object)
{
    /* Used to be able to hide the password while entering */
    static char passwd[65] = "";
    static int lastlength = 0;

    char *tempstrptr;
    int place, length, loginresult;
    Reply answer;

    firstpsw = passwd;
    if(object == loginpswinm) {
	place = SUIT_getInteger(object, CURSOR_INDEX);
	tempstrptr = SUIT_getText(object, CURRENT_VALUE);
	length = strlen(tempstrptr);
	if(length == lastlength) {
	} else if(length > lastlength) {
            /* write the entered character to the end of the buffer, hide */
	    strcat(passwd, tempstrptr+length-1);
	    tempstrptr[length-1] = '*';
	} else if(length < lastlength) {
	    if(place == length) {
                /* remove rightmost character in buffer */
		passwd[lastlength-1] = 0;
	    } else {
                /* Kill both buffer and stringgadget.string */
		passwd[0] = 0;
		tempstrptr[0] = 0;
		length = 0;
	    }
	}
	lastlength = length;
    } else if(object == loginokbutton) {
        /* Try to login */
	if(passwd == NULL) {
            GUI_error("Du mste ange ett lsenord!", 0);
            return;
        }
	if(strlen(passwd) == 0) {
	    GUI_error("Du mste ange ett lsenord!", 0);
	    return;
	}
	loginresult = lyx_login(SUIT_getText(loginservinm, CURRENT_VALUE), SUIT_getText(loginnamninm, CURRENT_VALUE), passwd);
	switch(loginresult) {
	  case LOGIN_OK:
	    break;
	  case NO_SERVER:
	    GUI_error("Kan inte finna server!", 0);
	    return;
	  case NAME_NOT_FOUND:
	    answer = SUIT_ask("Ingen anvndare med det namnet. Registrera/ngra namn ?", "Registrera", "ngra");
	    if(answer == REPLY_BUTTON1) {
		SUIT_setBoolean(loginBB, VISIBLE, FALSE);
		SUIT_setBoolean(dbcheckBB, VISIBLE, TRUE);
		SUIT_setText(dbuname, CURRENT_VALUE, SUIT_getText(loginnamninm, CURRENT_VALUE));
		firstpsw = passwd;
		return;
	    }
	    else
	      return;
	  case MORE_THAN_ONE_NAME_FOUND:
	    GUI_error("Var mer specifik! Mer n en anvndare med givet namn.", 0);
	    return;
	  case BAD_PASSWORD:
            /* Totally reinitialize password and try again */
	    GUI_error("Felaktigt lsenord!", 0);
	    passwd[0] = 0;
	    tempstrptr = SUIT_getText(loginpswinm, CURRENT_VALUE);
	    tempstrptr[0] = 0;
	    length = 0;
	    return;
	  default:
	    GUI_error("Intern fel!", 0);
	    return;
	}
	SUIT_setBoolean(loginBB, VISIBLE, FALSE);
	SUIT_setBoolean(mainBB, VISIBLE, TRUE);
    }
}

/* Callback for doublecheck/loginname, dbPswName */

void GUI_dbcheckdo(SUIT_object object)
{
    static char passwd[65] = "";
    static int lastlength = 0;

    char *tempstrptr;
    int place, length, loginresult;

    if(object != dbpsw)
      return;
    
    place = SUIT_getInteger(object, CURSOR_INDEX);
    tempstrptr = SUIT_getText(object, CURRENT_VALUE);
    length = strlen(tempstrptr);
    if(length == lastlength) {
    } else if(length > lastlength) {
	/* write the entered character to the end of the buffer, hide */
	strcat(passwd, tempstrptr+length-1);
	tempstrptr[length-1] = '*';
    } else if(length < lastlength) {
	if(place == length) {
	    /* remove rightmost character in buffer */
	    passwd[lastlength-1] = 0;
	} else {
	    /* Kill both buffer and stringgadget.string */
	    passwd[0] = 0;
	    tempstrptr[0] = 0;
	    length = 0;
	}
    }
    lastlength = length;
    secondpsw = passwd;
}

/* Callback for doublecheck/registrera, avbryt */

void GUI_dbregdo(SUIT_object object)
{
    if(secondpsw == NULL) {
	GUI_error("Du mste ange ett lsenord!", 0);
	return;
    }
    if(strlen(secondpsw) == 0) {
	GUI_error("Du mste ange ett lsenord!", 0);
	return;
    }
    if(object == dbokbutton) {
	if(strcmp(firstpsw, secondpsw) == NULL) {
	    lyx_create_user(SUIT_getText(dbuname, CURRENT_VALUE), secondpsw);
	    SUIT_setBoolean(mainBB, VISIBLE, TRUE);
	    SUIT_setBoolean(dbcheckBB, VISIBLE, FALSE);
	} else {
	    SUIT_setBoolean(loginBB, VISIBLE, TRUE);
	    SUIT_setBoolean(dbcheckBB, VISIBLE, FALSE);
	    GUI_error("Dina tv password var ej identiska!", 0);
	}
    } else {
	SUIT_setBoolean(loginBB, VISIBLE, TRUE);
	SUIT_setBoolean(dbcheckBB, VISIBLE, FALSE);
    }
}

/* Function to create and setup the doublelcheck window */

void GUI_create_dbcheck_win(void)
{
    SUIT_object dbnametxt, dbpswtxt;

    dbcheckBB = SUIT_createBulletinBoard("Password check window");
    dbnametxt = SUIT_createLabel("Vilket anvndarnamn?");
    dbuname = SUIT_createTypeInBox("DBLoginName", GUI_dbcheckdo);
    dbpswtxt = SUIT_createLabel("Verifiera password");
    dbpsw = SUIT_createTypeInBox("dbPswName", GUI_dbcheckdo);
    dbhelp = SUIT_createButton("Hjlp med verifiering", GUI_helpdo);
    dbokbutton = SUIT_createButton("Registrera", GUI_dbregdo);
    dbavbrytbutton = SUIT_createButton("Avbryt registrering", GUI_dbregdo);
    
    SUIT_addChildToObject(dbcheckBB, dbnametxt);
    SUIT_addChildToObject(dbcheckBB, dbuname);
    SUIT_addChildToObject(dbcheckBB, dbpswtxt);
    SUIT_addChildToObject(dbcheckBB, dbpsw);
    SUIT_addChildToObject(dbcheckBB, dbhelp);
    SUIT_addChildToObject(dbcheckBB, dbokbutton);
    SUIT_addChildToObject(dbcheckBB, dbavbrytbutton);

    /* To hide the password. */
    SUIT_setBoolean(dbpsw, ANY_KEYSTROKE_TRIGGERS, TRUE);
}

/* Function to create and setup the login window */

void GUI_create_login_win(void)
{
    SUIT_object loginservtxt, loginnamntxt, loginpswtxt, loginavbrytbutton;

    loginBB = SUIT_createBulletinBoard("LoginFnster");
    loginservtxt = SUIT_createLabel("Vilken server?");
    loginservinm = SUIT_createTypeInBox("ServerName", GUI_logindo);
    loginnamntxt = SUIT_createLabel("Ditt LysKOM-namn?");
    loginnamninm = SUIT_createTypeInBox("LoginName", GUI_logindo);
    loginpswtxt = SUIT_createLabel("Ditt LysKOM-password?");
    loginpswinm = SUIT_createTypeInBox("PswName", GUI_logindo);
    loginhelp = SUIT_createButton("Hjlp med login", GUI_helpdo);
    loginokbutton = SUIT_createButton("Logga in", GUI_logindo);
    loginavbrytbutton = SUIT_createButton("Avbryt inloggning", GUI_mavbrytdo);

    SUIT_addChildToObject(loginBB, loginavbrytbutton);
    SUIT_addChildToObject(loginBB, loginservtxt);
    SUIT_addChildToObject(loginBB, loginservinm);
    SUIT_addChildToObject(loginBB, loginnamntxt);
    SUIT_addChildToObject(loginBB, loginnamninm);
    SUIT_addChildToObject(loginBB, loginpswtxt);
    SUIT_addChildToObject(loginBB, loginpswinm);
    SUIT_addChildToObject(loginBB, loginhelp);
    SUIT_addChildToObject(loginBB, loginokbutton);

    /* To hide the password. */
    SUIT_setBoolean(loginpswinm, ANY_KEYSTROKE_TRIGGERS, TRUE);
}
