/* gui_userlist.c Functions concerning the userlist window in LyXKom */
/* version 0.2 */

#include <stdio.h>
#include <suit.h>
#include <lyxkom.h>
#include <gui_protos.h>
#include <lyx_protos.h>

/* The objects of the userlist window */
SUIT_object userlistBB, userlistlist, userlisthelp, userlistinfo;

enum whichlist {ONLIST, ALLLIST, NONE} whatamidoing = NONE;

extern lyx_who_is_on_return anvlista;

/* Function to write userinfo in textbox */

static void writeinfo(lyx_who_is_on_return *lista, int person)
{
    char TextBuffer[300];

    sprintf(TextBuffer, "%d. %s,\n@bold(r i:)\n%s,\n@bold(och gr:)\n%s", anvlista.number[person], anvlista.name[person], anvlista.conference[person], anvlista.doing[person]);
    SUIT_setText(userlistinfo, LABEL, TextBuffer);
}

/* Callback for "main/vrigt/vilkainloggade" */

void GUI_movilkado(SUIT_object object)
{
    int i;

    if(whatamidoing == ONLIST) {
	lyx_who_is_on_free(&anvlista);
    } else if(whatamidoing == ALLLIST) {
	lyx_list_all_users_free(&anvlista);
    }
    whatamidoing = ONLIST;
    lyx_who_is_on(&anvlista);
    /* Change all @ to @@ because of SUIT */
    for(i=0; i<anvlista.no_of_users; i++) {
	GUI_parse_at(&anvlista.name[i]);
	GUI_parse_at(&anvlista.conference[i]);
	GUI_parse_at(&anvlista.doing[i]);
    }
    SUIT_setTextList(userlistlist, LIST, SUIT_defTextList(anvlista.name, anvlista.no_of_users));
    writeinfo(&anvlista, 0);
    SUIT_setInteger(userlistlist, CURRENT_ROW, 0);
    GUI_toggle_visible(userlistBB);
}

/* Callback for "main/vrigt/lista anvndare" */

void GUI_molado(SUIT_object object)
{
    char ansstring[50];
    int i;

    if(SUIT_getString("Ange skvillkor. OK fr alla personer.", "", ansstring, 50) == REPLY_CANCEL) {
	return;
    }
    if(whatamidoing == ALLLIST) {
	lyx_list_all_users_free(&anvlista);
    } else if(whatamidoing == ONLIST) {
	lyx_who_is_on_free(&anvlista);
    }
    whatamidoing = ALLLIST;
    lyx_list_all_users(ansstring, &anvlista);
    for(i=0; i<anvlista.no_of_users; i++) {
	GUI_parse_at(&anvlista.name[i]);
    }
    SUIT_setTextList(userlistlist, LIST, SUIT_defTextList(anvlista.name, anvlista.no_of_users));
    SUIT_setText(userlistinfo, LABEL, "\n\n\n@centered(@bold(Tryck p \"Status anvndare\"))\n@centered(@bold(fr status))");
    SUIT_setInteger(userlistlist, CURRENT_ROW, 0);
    GUI_toggle_visible(userlistBB);
}

/* Callback for "anvlist/list". Checks for highlighted user and displays information */

void GUI_userlistlistdo(SUIT_object object)
{
    int choice;
    char TextBuffer[300];

    choice = SUIT_getInteger(object, CURRENT_ROW);
    if(choice < 0)
      GUI_error("Inget val!", 0);
    else if(whatamidoing == ONLIST) {
	writeinfo(&anvlista, choice);
    } else if(whatamidoing == ALLLIST) {
	SUIT_setText(userlistinfo, LABEL, "\n\n\n@centered(@bold(Tryck p \"Status anvndare\"))\n@centered(@bold(fr status))");
    }
}

/* Callback for "anvlist/status". Checks for highlited user and displays detailed information */

void GUI_userstatdo(SUIT_object object)
{
    lyx_status_user_return userstat;
    char TextBuffer[300], tottime[30];
    int choice, timebuff[4];
    
    choice = SUIT_getInteger(userlistlist, CURRENT_ROW);
    lyx_status_user(anvlista.number[choice], &userstat);
    GUI_parse_at(&userstat.host);
    if(choice < 0)
      GUI_error("Inget val!", 0);
    else {
	timebuff[0] = userstat.total_time;
	timebuff[1] = userstat.total_time % (3600*24);
	timebuff[2] = userstat.total_time % (3600);
        timebuff[3] = userstat.total_time % 60;

	sprintf(tottime, "%d dygn, %02d:%02d:%02d", timebuff[0]/(3600*24), timebuff[1]/3600, timebuff[2]/60, timebuff[3]);
	sprintf(TextBuffer,
		"@bold(Detaljerad anvndarinformation)\n\n@bold(Hemma-maskin:)%s\n@bold(Frsta inloggning:) %s, %s\n@bold(Senaste inloggning:) %s, %s\n@bold(Nrvarotid:) %s\n@bold(Skrivna meddelanden:) %d",
		userstat.host,
		userstat.first_login_date, userstat.first_login_time,
		userstat.last_login_date, userstat.last_login_time,
		tottime,
		userstat.num_written_messages);
	SUIT_setText(userlistinfo, LABEL, TextBuffer);
        lyx_status_user_free(&userstat);
    }
}

/* Function to create and setup the userlist window */

void GUI_create_userlist_win(void)
{
    char nulltext[] = "";
    SUIT_object userlistavbrbutton, userliststatbutton;

    userlistBB = SUIT_createBulletinBoard("UserListFnster");
    userlistlist = SUIT_createScrollableList("Username-list", GUI_userlistlistdo);
    userlistavbrbutton = SUIT_createButton("Avbryt anvndar-listning", GUI_avbrytdo);
    userliststatbutton = SUIT_createButton("Status anvndare", GUI_userstatdo);
    userlisthelp = SUIT_createButton("Hjlp med anvndar-listning", GUI_helpdo);
    userlistinfo = SUIT_createTextBox("UserInfo", nulltext);
    SUIT_addChildToObject(userlistBB, userlistlist);
    SUIT_addChildToObject(userlistBB, userlistavbrbutton);
    SUIT_addChildToObject(userlistBB, userliststatbutton);
    SUIT_addChildToObject(userlistBB, userlisthelp);
    SUIT_addChildToObject(userlistBB, userlistinfo);
}
