/* gui_utils.c Misc. functions used by other GUI-funktions in LyXKom. */
/* version 0.3 */

#include <stdlib.h>
#include <string.h>
#include <suit.h>
#include <lyxkom.h>
#include <gui_protos.h>
#include <lyxerr.h>


extern SUIT_object listBB, skrivBB, userlistBB;

/* This function makes a backup of an array of strings */

char **GUI_make_backup(char **strlist, int no_of)
{
    char **backup, *tmpptr;
    int i;

    if((backup = (char **) malloc(no_of*sizeof(char **))) == NULL)
      GUI_error("Kunde inte allokera minne!", 5);
    for (i=0; i<no_of; i++) {
	if((tmpptr = malloc(strlen(strlist[i])+1)) == NULL)
	  GUI_error("Kunde inte allokera minne!", 5);
	strcpy(tmpptr, strlist[i]);
	backup[i] = tmpptr;
    }
    return backup;
}

/* This function frees an array of strings */	

void GUI_free_backup(char **strlist, int no_of)
{
    int i;

    for(i=0; i<no_of; i++) {
	free(strlist[i]);
    }
    free(strlist);
}

/* Displays an error and waits for the user to confirm. If errorcod != 0, then it exits with errcode */

void GUI_error(char *errtxt, int errcode)
{
    SUIT_inform(errtxt);
    if(errcode == SERVER_ERROR)
      kom_perror();
    if(errcode)
      exit(errcode);
}

/* Counts how many 'sign' there is in 'string' */

int GUI_char_count(char *string, char sign)
{
    int count = 0;
    char *tempptr = string;

    while(tempptr = strchr(tempptr, sign)) {
        count++;
        tempptr++;
    }
    return count;
}

/* Erases a meeting from a lyx_list_meetings_return */

void GUI_erase_meeting(lyx_list_meetings_return *list, int which)
{
    char **tmpname;
    int *tmpnumber, *tmpunread;
    int no = list->no_of_meetings, i;

    if((tmpnumber = malloc((no-1) * sizeof(int))) == NULL)
       GUI_error("Kunde inte allokera minne!", 5);
    if((tmpunread = malloc((no-1) * sizeof(int))) == NULL)
      GUI_error("Kunde inte allokera minne!", 5);
    if((tmpname = malloc((no-1) * sizeof(char *))) == NULL)
      GUI_error("Kunde inte allokera minne!", 5);
    for(i=0; i<which; i++) {
	tmpnumber[i] = list->number[i];
	tmpunread[i] = list->unread[i];
	tmpname[i] = list->name[i];
    }
    for(i = which+1; i<no; i++) {
	tmpnumber[i-1] = list->number[i];
	tmpunread[i-1] = list->unread[i];
	tmpname[i-1] = list->name[i];
    }
    free(list->name[which]);
    free(list->name);
    free(list->number);
    free(list->unread);
    list->no_of_meetings--;
    list->name = tmpname;
    list->number = tmpnumber;
    list->unread = tmpunread;
}

/* Finds all @ and replaces with @@, the string is allocated with malloc() */

char *GUI_parse_at(char **stringptr)
{
    char *tempptr, *newstring, *startptr = *stringptr;
    int value, i;
    
    value = GUI_char_count(*stringptr, '@');
    if((newstring = (char *) malloc(value +strlen(*stringptr)+1)) == NULL) {
        GUI_error("Intern fel: Fr lite minne", 10);
    }
    if(value == 0) {
        strcpy(newstring, *stringptr);
	free(*stringptr);
	*stringptr = newstring;
	return newstring;
    }
    newstring[0] = 0;
    for(i=0; i<value; i++) {
        tempptr = strchr(startptr, '@');
        strncat(newstring, startptr, tempptr-startptr+1);
        strcat(newstring, "@");
        startptr = tempptr+1;
    }
    strcat(newstring, startptr);
    free(*stringptr);
    *stringptr = newstring;
    return newstring;
}

/* Inserts a string into another, and allocates place */

char * GUI_insert_string(char *string, char *insstring, int pos)
{
    char *newstring;
    int stringlength = strlen(string), insstringlength = strlen(insstring);

    if((newstring = (char *) malloc(stringlength + insstringlength +1)) == NULL) {
	GUI_error("Internt fel, GUI_insert_string:no memory", 5);
    }
    *newstring = 0;
    strncat(newstring, string, pos);
    strcat(newstring, insstring);
    strcat(newstring, string + pos);
    return(newstring);
}

/* Finds and returns the SUIT_object's root's child */

SUIT_object GUI_get_root_child(SUIT_object object)
{
    SUIT_object temp1, temp2;

    while((temp1 = SUIT_getParent(object)) != NULL) {
	temp2 = object;
	object = temp1;
    }
    return temp2;
}

/* Callback for all "avbryt" except main and login. Closes the actual window */

void GUI_avbrytdo(SUIT_object object)
{
    SUIT_object winobj;

    winobj = GUI_get_root_child(object);
    SUIT_setBoolean(winobj, VISIBLE, FALSE);
}

/* sets the SUIT_object visible and the others, which share the same space, to invisible */

void GUI_toggle_visible(SUIT_object object)
{
    SUIT_setBoolean(listBB, VISIBLE, FALSE);
    SUIT_setBoolean(skrivBB, VISIBLE, FALSE);
    SUIT_setBoolean(userlistBB, VISIBLE, FALSE);
    SUIT_setBoolean(object, VISIBLE, TRUE);
}


