/* gui_conflist.c Functions concerning the conference-list window in LyXKom. */
/* version 0.7 */

#include <stdio.h>
#include <suit.h>
#include <lyxkom.h>
#include <lyx_protos.h>
#include <gui_protos.h>
#include <stack.h>
#include <stack_protos.h>

/* Extern objects defined elsewhere in gui_* */

extern SUIT_object lasvar, laseditor;
extern Headerptr newstack, oldstack;


/* The objects of the conference list */
SUIT_object listradio, listlabel, listavbryt, listBB, liststack, listhelp, listlist, listbuttonstack, listtextbox;

lyx_list_meetings_return conflist;
lyx_status_meeting_return confstat;
SUIT_textList textlist = NULL;
enum type_of_list {SEEK, NEWS, MINE} listwhat;
char **namebackup;

/* Callback when you press listradio */

void GUI_liradiodo(SUIT_object object)
{
    char *text = NULL;

    text = SUIT_getEnumString(listradio, CURRENT_VALUE);
    if(strcmp(text, "G till") == 0)
      text = "Ange mte att g till";
    else if(strcmp(text, "Status") == 0)
      text = "Ange mte att ta status p";
    else if(strcmp(text, "Bli medlem i") == 0)
      text = "Ange mte att bli medlem i";
    else if(strcmp(text, "Uttrd ur") == 0)
      text = "Ange mte att uttrda ur";
    else {
        text = "Intern fel!";
        GUI_error("listradio: Internt fel!", 0);
    }
    SUIT_setText(listlabel, LABEL, text);
    SUIT_redisplayNotRequired(listBB);
    SUIT_redisplayRequired(listradio);
    SUIT_redisplayRequired(listlabel);
}

/* Function to setup the conference-list environment */

void GUI_conf_init(void )
{
    SUIT_setText(listtextbox, LABEL, "@centerd(@bold(Ange handling/mte))\n@centerd(@bold(eller))\n@centerd(@bold(begr Hjlp))");
    SUIT_pressThisRadioButton(listradio, "G till");
    GUI_liradiodo(listradio);
}

/* Function for defining the list of meetings */

void GUI_set_textlist(char **name, int no_of)
{
    if(textlist)
      SUIT_destroyTextList(textlist);
    textlist = SUIT_defTextList(name, no_of);
    SUIT_setTextList(listlist, LIST, textlist);
}

/* Callback for "ls/listamten/alla" */

void GUI_llallado(SUIT_object object)
{
    int i;
    char *tmpstr;
    char ansstring[50];

    if(SUIT_getString("Ange skvillkor. OK fr alla mten.", "", ansstring, 50) == REPLY_CANCEL) {
	return;
    }
    SUIT_performRedisplay();
    listwhat = SEEK;
    lyx_list_meetings(ansstring, &conflist);
    for(i=0; i<conflist.no_of_meetings; i++)
      GUI_parse_at(&conflist.name[i]);
    for (i=0; i<conflist.no_of_meetings; i++) {
	if(conflist.unread[i] != NOT_MEMBER) {
	    tmpstr = GUI_insert_string(conflist.name[i], "* ", 0);
	} else {
	    tmpstr = GUI_insert_string(conflist.name[i], "  ", 0);
	}
	free(conflist.name[i]);
	conflist.name[i] = tmpstr;
    }
    GUI_set_textlist(conflist.name, conflist.no_of_meetings);
    GUI_conf_init();
    SUIT_setText(listlist, LABEL, "@bold(medl.  Namn)                               ");
    GUI_toggle_visible(listBB);
}

/* Callback to "ls/listamten/mina" */

void GUI_llminado(SUIT_object object)
{
    int i;
    char *tmpstr, txtbuffer[30];

    listwhat = MINE;
    lyx_list_membership_meetings(&conflist);
    for(i=0; i<conflist.no_of_meetings; i++)
      GUI_parse_at(&conflist.name[i]);
    for(i=0; i<conflist.no_of_meetings; i++) {
	tmpstr = GUI_insert_string(conflist.name[i],
				   sprintf(txtbuffer, "%3d  ",conflist.unread[i]),
				   0);
	free(conflist.name[i]);
	conflist.name[i] = tmpstr;
    }
    GUI_set_textlist(conflist.name, conflist.no_of_meetings);
    GUI_conf_init();
    SUIT_setText(listlist, LABEL, "@bold(Olsta  Namn)                           ");
    GUI_toggle_visible(listBB);
}

/* Callback to "ls/listamten/nya" */

void GUI_llnyado(SUIT_object object)
{
    int i;
    char *tmpstr, txtbuffer[30];

    listwhat = NEWS;
    lyx_list_membership_news(&conflist);
    for(i=0; i<conflist.no_of_meetings; i++)
      GUI_parse_at(&conflist.name[i]);
    for(i=0; i<conflist.no_of_meetings; i++) {
	tmpstr = GUI_insert_string(conflist.name[i],
				   sprintf(txtbuffer, "%4d  ",conflist.unread[i]),
				   0);
	free(conflist.name[i]);
	conflist.name[i] = tmpstr;
    }
    GUI_set_textlist(conflist.name, conflist.no_of_meetings);
    GUI_conf_init();
    SUIT_setText(listlist, LABEL, "@bold(Olsta  Namn)                           ");
    GUI_toggle_visible(listBB);
}

/* Callback when you press listlist */

void GUI_lilistdo(SUIT_object object)
{
    char *whattodo, TextBuffer[300], mbuff[20], *meettype;
    int choice, cutlen, i=0, flag;

    choice = SUIT_getInteger(object, CURRENT_ROW);
    whattodo = SUIT_getEnumString(listradio, CURRENT_VALUE);
    if(strcmp(whattodo, "Bli medlem i") == 0) {
	if(lyx_subscribe(conflist.number[choice]) == SUCCESS) {
	    if(listwhat == SEEK) {
		*SUIT_itemInTextList(textlist, choice) = '*';
		SUIT_setTextList(listlist, LIST, textlist);
		conflist.unread[choice] = 0;
	    }
	}
    }
    else if(strcmp(whattodo, "Status") == 0) {
	lyx_status_meeting(conflist.number[choice], &confstat);
	switch(confstat.meeting_type) {
	  case MEETING_TYPE_CLOSED:
	    meettype = "Mtet r stngt.";
	    break;
	  case MEETING_TYPE_NO_COMMENTS:
	    meettype = "Mtet stngt fr kommentarer.";
	    break;
	  case MEETING_TYPE_MAILBOX:
	    meettype = "Mtet r en personlig brevlda.";
	    break;
	  case MEETING_TYPE_SECRET:
	    meettype = "Mtet r hemligt. Hur kom du hit egentligen? ;)";
	    break;
	  case MEETING_TYPE_NORMAL:
	    meettype = "Ett helt normalt mte.";
	    break;
	  default:
	    meettype = "Intet fel!";
	    GUI_error("Intern fel: mtestyp oknd", 0);Password:

	}
	GUI_parse_at(&confstat.creator);
	sprintf(TextBuffer, "@bold(Detaljerad mtesinformation:)\n\n@bold(Mtesnummer:) %d\n@bold(Skapare:) %s\n@bold(Skapad:) %s, %s\n@bold(Senast skrivet:) %s, %s\n@bold(Medlemmar:) %d personer\n@bold(Antal meddelanden:) %d stycken\n@bold(%s)\n",
		confstat.number,
		confstat.creator,
		confstat.create_date, confstat.create_time,
		confstat.last_written_date, confstat.last_written_time,
		confstat.num_members,
		confstat.num_of_messages,
		meettype);
	SUIT_setText(listtextbox, LABEL, TextBuffer);
    }
    else if(strcmp(whattodo, "G till") == 0) {
	destruct(newstack);
	newstack = construct();
	destruct(oldstack);
	oldstack = construct();
	if(conflist.unread[choice] != NOT_MEMBER) {
	    if(lyx_goto_meeting(conflist.number[choice]) == FAIL) {
		GUI_error("Det gick inte att g in i mtet!", 0);
		return;
	    }
	    SUIT_setText(laseditor, CURRENT_VALUE, "");
	    flag = 0;
	    while(isspace(conflist.name[choice][i])) {
		flag = 1;
		i++;
	    }
	    if (flag == 1)
	      i--;
	    cutlen = sscanf(conflist.name[choice]+i, "%s", mbuff);
	    SUIT_setText(lasvar, LABEL, conflist.name[choice]+cutlen+i);
	} else
	  GUI_error("Du mste bli medlem i mtet frst!", 0);
    }
    else if(strcmp(whattodo, "Uttrd ur") == 0) {
	if(lyx_unsubscribe(conflist.number[choice]) == SUCCESS) {
	    if(listwhat == SEEK) {
		*SUIT_itemInTextList(textlist, choice) = ' ';
		SUIT_setTextList(listlist, LIST, textlist);
	    } else {
		GUI_erase_meeting(&conflist, choice);
                SUIT_deleteFromTextList(textlist, choice);
		SUIT_setTextList(listlist, LIST, textlist);
	    }
	}
    }
    else {
        GUI_error("listradio: Internt fel!", 0);
    }
}

/* Function to create and setup the conference-list window */

void GUI_create_list_win(void)
{
    listBB = SUIT_createBulletinBoard("Listfnster");
    listbuttonstack = SUIT_createStacker("list button stacker");
    liststack = SUIT_createStacker("List stacker");
    listradio = SUIT_createRadioButtons("ListRadio", GUI_liradiodo);
    listlist = SUIT_createScrollableList("Mteslista", GUI_lilistdo);
    listtextbox = SUIT_createTextBox("ConfInfo", "");
    SUIT_addButtonToRadioButtons(listradio, "G till");
    SUIT_addButtonToRadioButtons(listradio, "Status");
    SUIT_addButtonToRadioButtons(listradio, "Bli medlem i");
    SUIT_addButtonToRadioButtons(listradio, "Uttrd ur");
    listlabel = SUIT_createLabel("Vlj mte/handling");
    listavbryt = SUIT_createButton("Avbryt lista", GUI_avbrytdo);
    listhelp = SUIT_createButton("Hjlp med lista", GUI_helpdo);
    SUIT_addChildToObject(liststack, listradio);
    SUIT_addChildToObject(listbuttonstack, listavbryt);
    SUIT_addChildToObject(listbuttonstack, listhelp);
    SUIT_addChildToObject(liststack, listbuttonstack);
    SUIT_addChildToObject(listBB, liststack);
    SUIT_addChildToObject(listBB, listlist);
    SUIT_addChildToObject(listBB, listlabel);
    SUIT_addChildToObject(listBB, listtextbox);
    SUIT_pressThisRadioButton(listradio, "G till");
}
