/* gui_write.c Functions concerning the write window in LyXKom. */
/* version 0.5 */

#include <string.h>

#include <suit.h>
#include <lyxkom.h>
#include <lyx_protos.h>
#include <gui_protos.h>
#include <lyxkom_global.h>

/* The objects of the write window */
SUIT_object skrivBB, skrivstack, skrivklar, skrivavbryt, skrivhelp, skrivedit;

enum writewhat {COMMENT, NEW} whatamidoing;
char subjstr[100];

/* Callback for "ls/skriv/svara" */

void GUI_lssvarado(SUIT_object object)
{
    if(last_text == 0) {
	GUI_error("Du mste lsa en text innan du kan kommentera!", 0);
	return;
    }
    whatamidoing = COMMENT;
    SUIT_setText(skrivedit, CURRENT_VALUE, "");
    GUI_toggle_visible(skrivBB);
}

/* Callback for "ls/skriv/frfatta" */

void GUI_lsnyttdo(SUIT_object object)
{
    char *instrstring = "Ange rendet:";
    int subok;
    
    if(curr_conf == 0) {
	GUI_error("Du mste g in i ett mte frst!", 0);
	return;
    }
    whatamidoing = NEW;
    do {
	subok = FALSE;
	if(SUIT_getString(instrstring, "", subjstr, 100) == REPLY_CANCEL)
	  return;
	if(strlen(subjstr) == 0) {
	    instrstring = "Du mste ange ett rende!";
	    subok = TRUE;
	}
    } while(subok == TRUE);
    SUIT_setText(skrivedit, CURRENT_VALUE, "");
    GUI_toggle_visible(skrivBB);
}

/* Callback for "skriv/klar" */

void GUI_sklardo(SUIT_object object)
{
    char *tmpstr, *sendstring;

    tmpstr = SUIT_getText(skrivedit, CURRENT_VALUE);
    
    if(whatamidoing == NEW) {
	sendstring = malloc(strlen(tmpstr) + strlen(subjstr) +2);
	sprintf(sendstring, "%s\n%s", subjstr, tmpstr);
	lyx_enter_new_text(curr_conf, sendstring);
	free(sendstring);
	GUI_error("Texten snd", 0);
	SUIT_setText(skrivedit, CURRENT_VALUE, "");
    } else {
	if(lyx_enter_comment_text(curr_conf, tmpstr) == 1)
	  GUI_error("Du kan inte kommentera i detta mtet!", 0);
	SUIT_setBoolean(skrivBB, VISIBLE, FALSE);
    }
}

/* Callback for "skriv/editor", invokes when user press ctrl-x */

void GUI_seditdo(SUIT_object object)
{
    SUIT_sendToEditor(skrivedit, "LyXkom ");
}

/* Function to create and setup the write window */

void GUI_create_write_win(void)
{
    skrivBB = SUIT_createBulletinBoard("SkrivFnster");
    skrivstack = SUIT_createStacker("Skrivstack");
    skrivedit = SUIT_createTextEditorWithScrollBar("SkrivEditor", GUI_seditdo);
    skrivklar = SUIT_createButton("Klar", GUI_sklardo);
    skrivavbryt = SUIT_createButton("Avbryt skriv", GUI_avbrytdo);
    skrivhelp = SUIT_createButton("Hjlp med Skriv", GUI_helpdo);
    SUIT_addChildToObject(skrivstack, skrivklar);
    SUIT_addChildToObject(skrivstack, skrivavbryt);
    SUIT_addChildToObject(skrivstack, skrivhelp);
    SUIT_addChildToObject(skrivBB, skrivstack);
    SUIT_addChildToObject(skrivBB, SUIT_name("SkrivEditor"));
}
