#ifndef LYXKOM_H
#define LYXKOM_H



#define SUCCESS 0
#define FAIL 1

/*
 * This is a return-structure from the lyx_who_is_on() to the gui-function
 * that is the caller.
 *
 * Just call lyx_who_is_on with a pointer a storage allocated for this
 * structure (normally done with an auto variable).
 * 
 */
 
typedef struct {
    int no_of_users;    /* Number of users currently logged on. */

    char **name;        /* An array of names for each user. */
    int  *number;       /* The number for each user */
    char **conference;  /* An array of where each user is right now. */
    char **doing;       /* An array of what each user is doing */
} lyx_who_is_on_return;

/*
 * Info about the message - if returned.
 *
 */

struct message_info {
    int  message_no;        /* The global number of the message */
    char *author;           /* Name of the author */
    int  no_of_lines;       /* Number of lines in message */
    int  is_comment_no;     /* Global text-num of the message that this
                            /* message is a comment to. */
    char *is_comment_auth;  /* The author of that message */
    int  num_comments;      /* Number of comments for this message */    

    char *date;             /* Date when message sent */
    char *time;             /* Time when message sent */

    char *text;             /* The actual text */
                            /* The subject is on the first line */
};

/*
 * Info about the meeting - if returned.
 *
 */

struct meeting_info {
    char *meeting_name;     /* Name of the meeting we're going to. */
    int  meeting_no;        /* The number of that meeting. */
    int  no_of_unread;      /* Number of unread texts in meeting. */
};

/*
 * The return from lys_get_next_text(), lys_get_prev_text() or
 * lys_get_spec_text().
 *
 * The flags holds info about what type we return. The data is stored
 * in the union type. For more info about the contents of that union,
 * see above.
 */

typedef struct {
    int flags;          /* End of messages, New meeting or New message */
    
    union {
        struct meeting_info meeting;
        struct message_info message;
    } type;
} lyx_get_text_return;

#define NOTHING_LEFT 0
#define MESSAGE_RETURN 1
#define MEETING_RETURN 2

/*
 * Return data from lyx_list_membership_news(), lyx_list_membership_meetings() and
 * lyx_list_meetings().
 *
 */

typedef struct {
    int  no_of_meetings;   /* Number of meetings returned */
    char **name;          /* An array of the name for each meeting */
    int  *number;         /* The number for each meeting */
    int  *unread;         /* How many unread messages for each meeting. -1 = not member */
} lyx_list_meetings_return;

#define NOT_MEMBER -1

/*
 *
 *
 */

typedef struct {
    char *name;              /* Name of meeting */
    int  number;             /* Number for this meeting */
    char *creator;           /* Name of the creator for this meeting */

    char *create_date;       /* Date when created */
    char *create_time;       /* Time when created */
    
    char *last_written_date; /* Date for last written message */
    char *last_written_time; /* Time for last written message */

    int  num_members;        /* Number of members in meeting */
    int  num_of_messages;    /* Number of messages in meeting */
    int  meeting_type;       /* Type of meeting */
} lyx_status_meeting_return;

#define MEETING_TYPE_CLOSED         1
#define MEETING_TYPE_NO_COMMENTS    2
#define MEETING_TYPE_MAILBOX        3
#define MEETING_TYPE_SECRET         4
#define MEETING_TYPE_NORMAL         0

/*
 * Returninfo from lyx_status_user().
 *
 *
 */

typedef struct {
    char *name;                /* Name of the user */
    int  number;               /* Number for this user */
    char *host;                /* Running from user@host */

    char *first_login_date;    /* First login */
    char *first_login_time;

    char *last_login_date;     /* Last login (or logout) */
    char *last_login_time;
    
    int  total_time;           /* Total time present (in seconds) */
    int  num_written_messages; /* Total number of written messages */
} lyx_status_user_return;

#endif




