#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <s-string.h>
#include <kom-types.h>
#include <kom-errno.h>

#include <lyxkom.h>
#include <lyx_protos.h>
#include <lyxerr.h>
#include <lyxkom_global.h>

#include "stack.h"
#include "stack_protos.h"

/*
 * NAME lyx_get_next_text
 *
 * CREATED           93-05-04 by Hez
 * LAST CHANGED:     93-05-17 by Hez
 *
 */

/*
 * This is the read-info-structure. It's used to keep track on what messages I've read in the
 * unread-list.
 */

struct Read_info {
    Text_no text;   /* Global text-no that is refered */
    int     flags;  /* Read state: UNREAD/READ */
};

#define TEXT_IS_UNREAD   0
#define TEXT_IS_READ     1

static Conf_no_list       meetings;            /* List of meetings that we have unread msgs in */
static int                curr_conf_list_pos;  /* Where we are in that list (to find next meeting */
static Conference         curr_conf_stat;      /* Status for the current meeting */

static Membership         membership;          /* Info about what I've read in current meeting */
static struct Read_info   *texts_to_read;      /* List of texts that I perhaps shall read */
static int                num_texts_to_read;   /* The size of the array above */
static Headerptr          read_stack;          /* Stack when reading comments */

char *last_text_subject;                       /* Holds subject for the last text read. 
                                                  Used in lyx_enter_comment_text() */

/*
 * These are the states that lyx_get_next_text go through.
 */

#define INIT_GOTO_NEXT_MEETING  0       /* First state - get a list of all meetings that are unread
                                           then goto the first meeting in that list */
#define GET_NEXT_TEXT           1       /* This is the main-state - here we read all texts that are
                                           in the unread-list and are not in the read-list. When 
                                           all texts are read we automatically go to next meeting,
                                           and sets the INIT_READ_TEXTS. If there aren't any meetings
                                           with unread texts the first state is set. */

static int read_state = INIT_GOTO_NEXT_MEETING;

/*
 * Prototypes for the local functions in this file.
 */

static void init_goto_next_meeting(void);
static int goto_next_meeting(lyx_get_text_return *);
static int is_text_already_read(Local_text_no);
static void mark_as_read(Text_no);
static Text_no find_next_text_to_read(int *);
static int get_next_text(lyx_get_text_return *);
static int cmp (Text_no *, Text_no *);
static void init_get_next_text(void);
static int get_text_data(Text_no, lyx_get_text_return *, int);

/* Defines for flags in get_text_data() */

#define NOT_GET_SPEC_TEXT  0
#define GET_SPEC_TEXT      1

/*
 * NAME lyx_goto_meeting
 *
 * CREATED           93-05-10 by Hez
 * LAST CHANGED:     93-05-19 by Hez
 *
 * DESCRIPTION
 *
 * This function changes the current meeting.
 * If you goto a meeting which is in the unread-meetings-list the lyx_get_next_text will start reading for last-read,
 * otherwise will lyx_get_next_text goto to the next meeting with unread texts and start reading there.
 *
 * This function always return SUCCESS - errors is handled via GUI_error.
 *
 */

int lyx_goto_meeting(int meeting_no)
{
    int i;

    if(read_state == INIT_GOTO_NEXT_MEETING || meeting_no==curr_conf)
        init_goto_next_meeting();

    for(i=0;i<meetings.no_of_confs;i++) {
        if(meetings.conf_nos[i]==meeting_no) {

/* Set the current conf */
            curr_conf_list_pos=i;
            curr_conf = meetings.conf_nos[curr_conf_list_pos];

/* Set pos to next conf in list */
            curr_conf_list_pos++;

/* Get status for this conference */
            curr_conf_stat = EMPTY_CONFERENCE;
            if(kom_get_conf_stat(curr_conf,&curr_conf_stat)==FAILURE)
                GUI_error("Serverfel (4:1)",SERVER_ERROR);

/* Get info about last text read and what other texts we have read */
            membership = EMPTY_MEMBERSHIP;
            if(kom_query_read_texts(user_id,curr_conf,&membership)==FAILURE)
                GUI_error("Serverfel (4:2)",SERVER_ERROR);

/* If we goto a meeting who have are unread texts, then the read-state must be set to read texts */
            kom_pepsi(curr_conf); /* Set curr-meeting in who_is_on on server */

/* Init so we start to read in this meeting. */
            init_get_next_text();

            return SUCCESS;
        }
    }

/* If we come to this point the user didn't want to go to a meeting with unread texts in. */
    curr_conf = meeting_no;
    curr_conf_stat = EMPTY_CONFERENCE;

/* Set so there is no last text to comment. */
    last_text = 0;

/* Kludge!!!!!!!! */
    if(curr_conf!=0) 
        if(kom_get_conf_stat(curr_conf,&curr_conf_stat)==FAILURE)
            GUI_error("Serverfel (4:3)",SERVER_ERROR);

    kom_pepsi(curr_conf);

/* Restart lyx_get_next_text if we goto a meeting that doesn't have any unread. */
    read_state = INIT_GOTO_NEXT_MEETING;

    return SUCCESS;
}

/*
 * NAME lyx_get_spec_text
 *
 * CREATED           93-05-10 by Hez
 * LAST CHANGED:     93-05-17 by Hez
 *
 * DESCRIPTION
 *
 * This is a very simple function - it only returns the text corresponding to text_no.
 * The text isn't marked as read!
 *
 */


void lyx_get_spec_text(lyx_get_text_return *ret, int text_no)
{
    get_text_data(text_no,ret,GET_SPEC_TEXT);       

    ret->flags=MESSAGE_RETURN;
}

/*
 * NAME lyx_get_next_text
 *
 * CREATED           93-05-04 by Hez
 * LAST CHANGED:     93-05-17 by Hez
 *
 * DESCRIPTION
 * Hez's log supplemental:
 * These are the functions of the LyXkom project, stardate 4023.4.
 *
 */

void lyx_get_next_text(lyx_get_text_return *ret)
{
    if(read_state == INIT_GOTO_NEXT_MEETING) {
        init_goto_next_meeting();    
        ret->flags = goto_next_meeting(ret);
        return;
    }

    if(read_state = GET_NEXT_TEXT) {
        ret->flags = get_next_text(ret);
        return;
    }
/* We never come here. */
}

/*
 * NAME init_goto_meeting
 *
 * CREATED           93-05-04 by Hez
 * LAST CHANGED:     93-05-19 by Hez
 *
 * DESCRIPTION
 *
 */

static void init_goto_next_meeting(void)
{
    last_text = 0;
    curr_conf_list_pos = 0;
    meetings = EMPTY_CONF_NO_LIST;
    if(kom_get_unread_confs(user_id,&meetings)==FAILURE)
        GUI_error("Serverfel (4:4)",SERVER_ERROR);
    return;
}

/*
 * NAME goto_next_meeting
 *
 * CREATED           93-05-04 by Hez
 * LAST CHANGED:     93-05-19 by Hez
 *
 * DESCRIPTION
 *
 */

static int goto_next_meeting(lyx_get_text_return *ret)
{

/* Check if we are in the end of the unread-meetings-list */
    if(curr_conf_list_pos>=meetings.no_of_confs) {

/* If we already are in INIT_GOTO_NEXT_MEETING state and try to go to the
   next meeting, then give error */
        if(read_state == INIT_GOTO_NEXT_MEETING) {

/* Clear some globals */
            curr_conf = 0;
            last_text = 0;
            kom_pepsi(0);
            return NOTHING_LEFT;
        }
/* Otherwise check if there are any unread meetings and try again. */
        read_state = INIT_GOTO_NEXT_MEETING;
        init_goto_next_meeting();

        return goto_next_meeting(ret);
    }

    curr_conf = meetings.conf_nos[curr_conf_list_pos]; /* Get next meeting-no */
    curr_conf_list_pos++; /* Update pointer */

/* Get stats for the new current meeting */
    curr_conf_stat = EMPTY_CONFERENCE;
    if(kom_get_conf_stat(curr_conf,&curr_conf_stat)==FAILURE)
        GUI_error("Serverfel (4:5)",SERVER_ERROR);

/* Get info about last text read and what other texts we have read */
    membership = EMPTY_MEMBERSHIP;
    if(kom_query_read_texts(user_id,curr_conf,&membership)==FAILURE)
        GUI_error("Serverfel (4:6)",SERVER_ERROR);

/* Check if there really are any new messages here */
    if(curr_conf_stat.no_of_texts<=0 ||
       curr_conf_stat.first_local_no + curr_conf_stat.no_of_texts 
     < membership.last_text_read)
        return goto_next_meeting(ret);

    kom_pepsi(curr_conf); /* Tell server that we have changed meeting */

/* Set the return structure for the GUI-functions */
    ret->type.meeting.meeting_name = s_crea_c_str(curr_conf_stat.name);
    ret->type.meeting.meeting_no   = curr_conf;
    ret->type.meeting.no_of_unread = curr_conf_stat.first_local_no + 
     curr_conf_stat.no_of_texts - membership.last_text_read - 1;

/* Ok, we've gone to a meeting - start to read texts */
    init_get_next_text();

/* Return meeting */
    return MEETING_RETURN;
}

/*
 * NAME init_get_next_text
 *
 * CREATED           93-05-04 by Hez
 * LAST CHANGED:     93-05-19 by Hez
 *
 * DESCRIPTION
 *
 */

static void init_get_next_text(void)
{
    Text_list    texts         = EMPTY_TEXT_LIST;
    int          i;

/* Create the comment stack */
    read_stack=construct();

/* Calculate how many messages we shall read */
    num_texts_to_read = curr_conf_stat.first_local_no+curr_conf_stat.no_of_texts
     -membership.last_text_read-1;

/* The the text-nos of every text that we (perhaps) shall read */
    if(kom_get_map(curr_conf,membership.last_text_read+1,num_texts_to_read,&texts)==FAILURE)
        GUI_error("Ofarligt fel (4:7)",NOT_REALLY_ERROR);

    num_texts_to_read=texts.no_of_texts;

/* Sort that list */
    qsort(texts.texts,num_texts_to_read,sizeof(Text_no),cmp);

/* Create our global list from that text-list */    
    texts_to_read=malloc(num_texts_to_read*sizeof(struct Read_info));
    for(i=0;i<num_texts_to_read;i++) {
         texts_to_read[i].text = texts.texts[i];
         texts_to_read[i].flags = TEXT_IS_UNREAD;
    }

    last_text = 0;

    read_state = GET_NEXT_TEXT;
}

/*
 * This is the comp-callback from qsort().
 */

static int cmp (Text_no *arg1, Text_no *arg2)
{
    if(*arg1 == *arg2)
        return 0;

    if(*arg1 > *arg2)
         return 1;

    return -1;
}

/*
 * NAME is_text_already_read
 *
 * CREATED           93-05-04 by Hez
 * LAST CHANGED:     93-05-17 by Hez
 *
 * DESCRIPTION
 * Returns: SUCCESS if text is already read, and
 *          FAIL if it's unread
 */

static int is_text_already_read(Local_text_no text_no)
{
    int top=0, bottom, mid;
    int found=-1;

    bottom = membership.no_of_read;

/* Return unread if we haven't read any after last read. */
    if(bottom==0)
        return FAIL; 

/* Binary search */
    while(top<=bottom) {
       mid = (top+bottom) /2;
       if(membership.read_texts[mid] == text_no) {
          found = mid;
          break;
       }
       else if(membership.read_texts[mid] > text_no)
          bottom = mid-1;
       else
          top = mid+1;
    }

    return (found==-1)?FAIL:SUCCESS;
}

/*
 * NAME mark_as_read
 *
 * CREATED           93-05-04 by Hez
 * LAST CHANGED:     93-05-17 by Hez
 *
 * DESCRIPTION
 *
 */

static void mark_as_read(Text_no text_no)
{
    int top=0, bottom, mid;
    int found=-1;

    bottom = num_texts_to_read;

/* Binary search */
    while(top<=bottom) {
       mid = (top+bottom) /2;
       if(texts_to_read[mid].text == text_no) {
          found = mid;
          break;
       }
       else if(texts_to_read[mid].text > text_no)
          bottom = mid -1;
       else
          top = mid +1;
    }

    if(found >= 0)
        texts_to_read[found].flags = TEXT_IS_READ;
}

/*
 * NAME get_text_data
 *
 * CREATED           93-05-04 by Hez
 * LAST CHANGED:     93-05-18 by Hez
 *
 * DESCRIPTION
 * Returns SUCCESS if text is read - and control should go back to GUI, and FAIL if the text_no
 * wasn't a real message.
 */

static int get_text_data(Text_no text_no, lyx_get_text_return *ret, int flags)
{
    String        text          = EMPTY_STRING;
    Text_stat     text_stat     = EMPTY_TEXT_STAT;
    Conference    tmp_conf_stat = EMPTY_CONFERENCE;
    Text_stat     tmp_text_stat = EMPTY_TEXT_STAT;
    Local_text_no local_no      = 0;
    Conf_no       rcpt_conf     = 0;  
    int           num_miscs     = 0;
    int           i;
    char          *tmp;

/* Null some members */
    ret->type.message.num_comments = 0;
    ret->type.message.is_comment_no = 0;
    ret->type.message.is_comment_auth = malloc(1);
    *ret->type.message.is_comment_auth = 0; /* Make an empty string */

    if (flags == NOT_GET_SPEC_TEXT)
        mark_as_read(text_no);

    if (kom_get_text_stat(text_no,&text_stat) == FAILURE)
        return FAIL;
    if (kom_get_text(text_no,0,END_OF_STRING,&text) == FAILURE)
        return FAIL;

/* Check if the text is in the texts-read-list */
    for(i=0;i<text_stat.no_of_misc;i++) {
        switch( text_stat.misc_items[i].type ) {
            case loc_no: 
                local_no=text_stat.misc_items[i].datum.local_no;
                num_miscs++;
                break;

            case recpt:
            case cc_recpt:
                rcpt_conf=text_stat.misc_items[i].datum.recipient;
                num_miscs++;
                break;
        }

        if(num_miscs % 2 == 0 && rcpt_conf == curr_conf) {
           if(kom_mark_as_read(rcpt_conf,1,&local_no)==FAILURE)
               GUI_error("Kunde ej markera texten som lst",NOT_REALLY_ERROR);
           break; /* End loop when both are found */
        }
    }

    if(flags == NOT_GET_SPEC_TEXT)
        if( is_text_already_read( local_no) == SUCCESS)
            return FAIL;

/* Fill in the message_no member */
    ret->type.message.message_no = text_no;

/* Fill in the author member */
    kom_get_conf_stat(text_stat.author,&tmp_conf_stat);
    ret->type.message.author = s_crea_c_str(tmp_conf_stat.name);

/* Fill in the no_of_lines member */
    ret->type.message.no_of_lines = text_stat.no_of_lines;

    for(i=0;i<text_stat.no_of_misc;i++) {
       Info_type temp1=text_stat.misc_items[i].type;
       Info_datum temp2=text_stat.misc_items[i].datum;

       switch(temp1) {

/* Fill in the is_comment_no & is_comment_auth members */
           case comm_to:
              ret->type.message.is_comment_no = temp2.comment_to;
              kom_get_text_stat(temp2.comment_to,&tmp_text_stat);
              kom_get_conf_stat(tmp_text_stat.author,&tmp_conf_stat);
              ret->type.message.is_comment_auth =
               s_crea_c_str(tmp_conf_stat.name);
              break;

/* Fill in the num_comments member */
           case comm_in:
              if(flags == NOT_GET_SPEC_TEXT)
                  push(&temp2.commented_in,sizeof(temp2.commented_in),
                   read_stack);
              ret->type.message.num_comments++;
              break;
        }
    }

/* Fill in the date and time members */
    ret->type.message.date = malloc(9);
    sprintf(ret->type.message.date, "%02d-%02d-%02d",
     text_stat.creation_time.tm_year,
     text_stat.creation_time.tm_mon+1, 
     text_stat.creation_time.tm_mday);
    ret->type.message.time = malloc(6);
    sprintf(ret->type.message.time, "%02d:%02d",
     text_stat.creation_time.tm_hour,
     text_stat.creation_time.tm_min);
       
/* Fill in the text member. The first line is subject. */
    ret->type.message.text = s_crea_c_str(text);

/* Set last_text_read global needed if we want to comment this text */
    last_text = text_no;

/* Save the subject of the last text read in a global string */
    last_text_subject = strdup(ret->type.message.text);

/* Alloc space for new text with '=====' - line */
    free(ret->type.message.text);
    ret->type.message.text = malloc(strlen(last_text_subject)+100);

/* Find the subject */
    if((tmp=strpbrk(last_text_subject,"\n\r"))!=NULL)
        *tmp=0;

/* Make the new text with '=====' */
    tmp++;
    sprintf(ret->type.message.text,"%s\n=======================================================================\n%s\n",
     last_text_subject,tmp);

    return SUCCESS;
}

/*
 * NAME find_next_text_to_read
 *
 * CREATED           93-05-04 by Hez
 * LAST CHANGED:     93-05-17 by Hez
 *
 * DESCRIPTION
 *
 */

static Text_no find_next_text_to_read(int *end_of_read)
{
    int i=0;

/* Linear search for the first pos in array which have member text != 0
   and member flags == TEXT_IS_UNREAD */
    while( (i < num_texts_to_read) &&
           (texts_to_read[i].text == 0) || 
           (texts_to_read[i].flags == TEXT_IS_READ) )
        i++;

/* If we ran out of list - notify caller */
    if(i >= num_texts_to_read) {
        kom_set_unread(curr_conf,0);
        *end_of_read=1;    
    }

    return texts_to_read[i].text;
}

/*
 * NAME get_next_text
 *
 * CREATED           93-05-04 by Hez
 * LAST CHANGED:     93-05-17 by Hez
 *
 * DESCRIPTION
 *
 */

static int get_next_text(lyx_get_text_return *ret)
{
    int text_to_read;
    int result;
    int end_of_read=0;

    do
    {
        if(empty(read_stack)) /* Any comments left to read? */
            text_to_read=find_next_text_to_read(&end_of_read);
        else
            text_to_read = *(Text_no *)pop(read_stack);

/* If there isn't any messages left in this meeting - go to next meeting */
        if(end_of_read==1)
            return goto_next_meeting(ret);

/* Make the return-structure */
        result=get_text_data(text_to_read, ret, NOT_GET_SPEC_TEXT);
    }
    while(result==FAIL);

/* Return a message */
    return MESSAGE_RETURN;
}

