#include <stdio.h>
#include <stdlib.h>

#include <s-string.h>
#include <kom-types.h>
#include <kom-errno.h>
#include <services.h>

#include <lyxkom.h>
#include <lyx_protos.h>
#include <lyxerr.h>
#include <lyxkom_global.h>
#include <gui_protos.h>

/*
 * NAME lyx_list_all_users.c
 *
 * CREATED           93-05-13 by Janne
 * LAST CHANGED:     93-05-18 by Ajo
 *
 * DESCRIPTION
 *
 * Call this function with a search string and a ptr to an allocated 
 * lyx_who_is_on_return-structure.
 *
 * This function fills it with following info for each meeting:
 *   
 *   Users names
 *   Users numbers
 *   
 * All space will be allocated by this function, so don't forget to call
 * lyx_list_all_users_free() when you don't need the information more.
 *
 */

void lyx_list_all_users(char *search_str,lyx_who_is_on_return *return_struct)
{
    
    int i;
    int j=0;

    Conf_list conf_list = EMPTY_CONF_LIST;
    String search_Str = EMPTY_STRING;
    Conference conference = EMPTY_CONFERENCE;
    
    /* Make 'String' of ordinary c-string*/

    if (s_crea_str(&search_Str,search_str) == FAILURE)
      GUI_error("Minnet slut (5:1)",MEMORY_ERROR);
    
    /* Lookup all the meetings and store them in conf_list */

    if (kom_lookup_name(search_Str, &conf_list) == FAILURE) 
      GUI_error("Serverfel (5:2)", SERVER_ERROR);

    /* Allocate memory for the posts in the return_struct */
    return_struct->number = malloc(conf_list.no_of_confs*sizeof(int));
    return_struct->name  = malloc(conf_list.no_of_confs*sizeof(char*));
        
    if(return_struct->name == NULL || return_struct->number == NULL)
      GUI_error("Minnet slut (5:3)",MEMORY_ERROR);
    
    /* Here starts the loop which fills the return_struct with number and
       name of all the search meetings */

    for (i=0; i<conf_list.no_of_confs; i++) {

        /* Pick out the meetings which are personally mailboxes */

        if(conf_list.confs[i].type.letter_box) {
            
            /* Get the number of each user/mailbox */

            return_struct->number[j] = conf_list.confs[i].conf_no;
            
            /* Get the name of each user/mailbox */

            if (kom_get_conf_stat(conf_list.confs[i].conf_no, 
                                  &conference) == FAILURE)
              GUI_error("Serverfel (5:4)",SERVER_ERROR);
	    
	    return_struct->name[j] = s_crea_c_str(conference.name);
            if(return_struct->name[j] == NULL)
              GUI_error("Minnet slut (5:5)",MEMORY_ERROR);
            j++;
	}
    }
    
    /* Fill the structure with no_of_users */
    return_struct->no_of_users = j;
}


void lyx_list_all_users_free(lyx_who_is_on_return *return_struct)
{
    int i;

    for(i=0; i<return_struct->no_of_users; i++) {
	free(return_struct->name[i]);
    }
    free(return_struct->name);
    free(return_struct->number);
    memset(return_struct,sizeof(lyx_who_is_on_return),0);
}
