#include <stdio.h>
#include <stdlib.h>

#include <s-string.h>
#include <kom-types.h>
#include <kom-errno.h>
#include <services.h>

#include <lyxkom.h>
#include <lyx_protos.h>
#include <lyxerr.h>
#include <lyxkom_global.h>
#include <gui_protos.h>

/*
 * NAME lyx_list_meetings.c
 *
 * CREATED           93-05-12 by Janne
 * LAST CHANGED:     93-05-18 by Ajo
 *
 * DESCRIPTION
 *
 * Call this function with a search string and a ptr to an allocated 
 * lyx_list_meetings_return-structure.
 *
 * This function fills it with following info for each meeting:
 * 
 * - Name of meeting
 * - Number of meetings
 * - Type of meeting (Closed, No comments, secret, normal)
 * - The number for each meeting 
 * - Number of unread texts for each meeting 
 *
 * All space will be allocated by this function, so don't forget to call
 * lyx_list_meetings_free() when you don't need the information more.
 *
 */

void lyx_list_meetings(char *search_str, 
		       lyx_list_meetings_return *return_struct)
{
    int i,n;
    int j=0;
    Conf_list conf_list = EMPTY_CONF_LIST;
    Conference conference = EMPTY_CONFERENCE;
    String search_Str = EMPTY_STRING;
    
    Person user_info = EMPTY_PERSON;
    Membership_list  test_list = EMPTY_MEMBERSHIP_LIST;

    /* Get person-status to find out how many meetings the person
       is a member in */
     
    if (kom_get_person_stat((Pers_no)user_id,&user_info) == FAILURE)
      GUI_error("Serverfel (6:1)",SERVER_ERROR);
    
    /* Get information about the person's memberships */
   
    if (kom_get_membership((Pers_no)user_id,0,user_info.no_of_confs+1,1,
                           &test_list) == FAILURE)
      GUI_error("Serverfel (6:2)",SERVER_ERROR);

    /* Make 'String' of ordinary c-string*/

    if (s_crea_str(&search_Str,search_str) == FAILURE)
      GUI_error("Minnet slut (6:3)",MEMORY_ERROR);

    /* Lookup all the meetings that matches the search-string 
       (blank string gives all the meetings) and store them in conf_list */

    if (kom_lookup_name(search_Str, &conf_list) == FAILURE) 
      GUI_error("Serverfel (6:4)", SERVER_ERROR);
    

    /* Allocate memory for the posts in the return_struct */

    return_struct->number = malloc(conf_list.no_of_confs*sizeof(int));
    return_struct->name  = malloc(conf_list.no_of_confs*sizeof(char*));
    return_struct->unread = malloc(conf_list.no_of_confs*sizeof(int));

    if(return_struct->name == NULL || return_struct->number == NULL ||
       return_struct->unread == NULL)
      GUI_error("Minnet slut (6:5)",MEMORY_ERROR);


    /* Here starts the loop which fills the return_struct with number and
       name of all the search meetings */

    for (i=0; i<conf_list.no_of_confs; i++) {

	/* Skip all meetings that actually are personally mailboxes */

	if(!conf_list.confs[i].type.letter_box) {
	    
	    /* Get the number of each conference */
	    return_struct->number[j] = conf_list.confs[i].conf_no;
	    
	    /* Get the name of each conference */
	    if (kom_get_conf_stat(conf_list.confs[i].conf_no, 
				  &conference) == FAILURE)
	      GUI_error("Serverfel (6:6)",SERVER_ERROR);
	    
	    return_struct->name[j] = s_crea_c_str(conference.name);
	    if(return_struct->name[j] == NULL)
	      GUI_error("Minnet slut (6:7)",MEMORY_ERROR);
	    
	    /* Check if member, set unread = -1 if not member */
	    
	    for (n=0; n<user_info.no_of_confs; n++) { 
		if (conf_list.confs[i].conf_no ==test_list.confs[n].conf_no) { 
		    return_struct->unread[j] = 0;
		    break;
		}
		else 
		  return_struct->unread[j] = -1;
	    }
	    j++;
	}
	
	    
    }
    /* Get the total number of searched meetings */

    return_struct->no_of_meetings = j;

}



	    
