#include <stdio.h>
#include <stdlib.h>

#include <s-string.h>
#include <kom-types.h>
#include <kom-errno.h>
#include <services.h>

#include <lyxkom.h>
#include <lyx_protos.h>
#include <lyxerr.h>
#include <lyxkom_global.h>
#include <gui_protos.h>

/*
 * NAME lyx_list_membership_meetings.c
 *
 * CREATED           93-05-11 by Janne
 * LAST CHANGED:     93-05-18 by Janne
 *
 * DESCRIPTION
 *
 * Call this function with a ptr to an allocated 
 * lyx_list_meetings_return-structure.
 *
 * This function fills it with following info for each meeting:
 *   
 * - Name of meeting
 * - Number of meetings
 * - Type of meeting (Closed, No comments, secret, normal)
 * - The number for each meeting 
 * - Number of unread texts for each meeting 
 *
 * All space will be allocated by this function, so don't forget to call
 * lyx_list_meetings_free() when you don't need the information more.
 *
 */

void lyx_list_membership_meetings(lyx_list_meetings_return *return_struct)
{
    int i;
    
    Person user_info = EMPTY_PERSON;
    Membership_list membership_list = EMPTY_MEMBERSHIP_LIST;
    Conference conference = EMPTY_CONFERENCE;
    Membership membership = EMPTY_MEMBERSHIP;
    
    /* Get person-status to find out how many meetings the person
       is a member in */
    if (kom_get_person_stat((Pers_no)user_id,&user_info) == FAILURE)
      GUI_error("Serverfel (7:1)",SERVER_ERROR);
    /* Get information about the person's memberships */
    if (kom_get_membership((Pers_no)user_id,0,user_info.no_of_confs+1,1,
			   &membership_list) == FAILURE)
      GUI_error("Serverfel (7:2)",SERVER_ERROR);
    
    /* Get number of unread confs */
    
    return_struct->no_of_meetings = user_info.no_of_confs;
    
    /* Allocate memory for the posts in the return_struct */
    
    return_struct->name   = malloc(user_info.no_of_confs*sizeof(char *));
    return_struct->number = malloc(user_info.no_of_confs*sizeof(int));
    return_struct->unread = malloc(user_info.no_of_confs*sizeof(int));
    
    if(return_struct->name == NULL || return_struct->number == NULL || 
       return_struct->unread == NULL)
      GUI_error("Minnet slut (7:3)",MEMORY_ERROR);

    /* Here starts the loop which fills the return_struct with number and
       name of all the person's membership meetings */
    
    for (i=0; i<user_info.no_of_confs; i++) {
	
	/* Get the number of each conference */
	
	return_struct->number[i] = membership_list.confs[i].conf_no;
	
	/* Get the name of each conference */
	
	if (kom_get_conf_stat(membership_list.confs[i].conf_no, 
			      &conference) == FAILURE)
	  GUI_error("Serverfel (7:4)",SERVER_ERROR);
	
	return_struct->name[i] = s_crea_c_str(conference.name);
	if(return_struct->name[i] == NULL)
	  GUI_error("Minnet slut (7:5)",MEMORY_ERROR);
    
	/* Get number of unread messages in each conference */

	if(conference.no_of_texts < 1)
          return_struct->unread[i] = 0;
        else {
	    if (kom_query_read_texts(user_id,membership_list.confs[i].conf_no, 
				     &membership) == FAILURE)
	      GUI_error("Serverfel (7:6)",SERVER_ERROR);
	    return_struct->unread[i] = conference.first_local_no + 
	      conference.no_of_texts - membership.last_text_read - 
		membership.no_of_read-1;
	}
    }
}





