#include <stdio.h>
#include <stdlib.h>

#include <s-string.h>
#include <kom-types.h>
#include <kom-errno.h>
#include <services.h>

#include <lyxkom.h>
#include <lyx_protos.h>
#include <lyxerr.h>
#include <gui_protos.h>
#include <lyxkom_global.h>


/*
 * NAME lyx_list_membership_news.c
 *
 * CREATED           93-05-05 by Ajo
 * LAST CHANGED:     93-05-18 by Janne
 *
 * DESCRIPTION
 *
 * Call this function with a ptr to an allocated lyx_list_meetings_return-structure.
 * This function fills it with following info for each meeting:
 *   
 * - Name of meeting
 * - Number of meetings
 * - Type of meeting (Closed, No comments, secret, normal)
 * - The number for each meeting 
 * - Number of unread texts for each meeting 
 *
 * All space will be allocated by this function, so don't forget to call
 * lyx_list_meetings_free() when you don't need the information more.
 *
 */
  
void lyx_list_membership_news(lyx_list_meetings_return *return_struct)
{
    int i;
    
    Conf_no_list conflist = EMPTY_CONF_NO_LIST;
    Conference conference = EMPTY_CONFERENCE;
    Membership membership = EMPTY_MEMBERSHIP;
    
    if (kom_get_unread_confs(user_id, &conflist) == FAILURE)
      GUI_error("Serverfel (8:1)",SERVER_ERROR);
    
    /* Get number of unread confs */
    
    return_struct->no_of_meetings = conflist.no_of_confs;
    
    /* Allocate memory for the posts in the return_struct */
    
    return_struct->name = malloc(conflist.no_of_confs*sizeof(char *));
    return_struct->number = malloc(conflist.no_of_confs*sizeof(int));
    return_struct->unread = malloc(conflist.no_of_confs*sizeof(int));
    
    if(return_struct->name == NULL || return_struct->number == NULL || return_struct->unread == NULL)
      GUI_error("Minnet slut (8:2)",MEMORY_ERROR);
    
    for (i=0; i<conflist.no_of_confs; i++) {
	
	/* Get the number of each conference */
	
        return_struct->number[i] = conflist.conf_nos[i];
	
	/* Get the name of each conference */
	
	if (kom_get_conf_stat(conflist.conf_nos[i], &conference) == FAILURE)
	  GUI_error("Serverfel (8:3)",SERVER_ERROR);
        return_struct->name[i] = s_crea_c_str(conference.name);
        if(return_struct->name[i] == NULL)
	  GUI_error("Minnet slut (8:4)",MEMORY_ERROR);
	
	/* Get number of unread messages in each conference */
	
	if(conference.no_of_texts < 1)
          return_struct->unread[i] = 0;
        else {
	    if (kom_query_read_texts(user_id, conflist.conf_nos[i],
				     &membership) == FAILURE)
	      GUI_error("Serverfel (8:5)",SERVER_ERROR);
	    return_struct->unread[i] = conference.first_local_no +
	      conference.no_of_texts - membership.last_text_read -
		membership.no_of_read - 1;
	}
    }
}

 
void lyx_list_meetings_free(lyx_list_meetings_return *return_struct)
{
    int i;
    
    for(i=0; i<return_struct->no_of_meetings; i++) {
        free(return_struct->name[i]);
    }
    
    free(return_struct->name);
    free(return_struct->number);
    free(return_struct->unread);
    memset(return_struct,sizeof(lyx_list_meetings_return),0);
}

