#include <stdio.h>
#include <stdlib.h>

#include <s-string.h>
#include <kom-types.h>
#include <kom-errno.h>
#include <services.h>

#include <lyxkom.h>
#include <lyx_protos.h>
#include <lyxerr.h>
#include <lyxkom_global.h>

#include <gui_protos.h>

/*
 * NAME lyx_status_meeting.c
 *
 * CREATED           93-05-10 by Janne
 * LAST CHANGED:     93-05-18 by Janne
 *
 * DESCRIPTION
 *
 * Call this function with a ptr to an allocated lyx_status_meeting_return-structure.
 * This function fills it with following info for a specified meeting:
 *   
 * -Name of meeting
 * -Number for this meeting
 * -Name of the creator for this meeting
 * -Date and time when meeting created
 * -Date and time for last written message
 * -Number of members in meeting
 * -Number of messages in meeting
 * -Type of meeting (Closed, No comments, secret, normal)
 *
 * All space will be allocated by this function, so don't forget to call
 * lyx_status_meeting_free() when you don't need the information more.
 *
 */

void lyx_status_meeting(int meeting_number, lyx_status_meeting_return *return_struct)      
{
    Conference conf_info = EMPTY_CONFERENCE;
    Conference creator_info = EMPTY_CONFERENCE;
    
    /* Call the low-level functions to get informatiotion to process */
    
    if (kom_get_conf_stat((Conf_no)meeting_number, &conf_info) == FAILURE)      
      GUI_error("Serverfel (11:1)",SERVER_ERROR);
    
    /* Get the name of the meeting */
    
    return_struct->name = s_crea_c_str(conf_info.name);

    if(return_struct->name == NULL)
      GUI_error("Minnet slut (11:2)",MEMORY_ERROR);
    
    /* Put the argument 'meeting_number' in the struct */
    
    return_struct->number = meeting_number;
    
    /* Get the name of the meeting's creator */
    
    if (kom_get_conf_stat((Conf_no)conf_info.creator, &creator_info) == FAILURE) {  
	return_struct->creator = malloc(20);
	strcpy(return_struct->creator,"ej ngon skapare");
    }
    else {
	return_struct->creator = s_crea_c_str(creator_info.name);
	if(return_struct->creator == NULL)
	  GUI_error("Minnet slut (11:3)",MEMORY_ERROR);
    }
    
    /* Get the date and time when meeting created  *
       
     * By some strange, and to us unknown reason,  *
     * the 'month' is 1 less than it should be, so *
     * the '+1' is just a way to get it allright...*/
    
    return_struct->create_date = malloc(9);
    sprintf(return_struct->create_date,"%02d-%02d-%02d", conf_info.creation_time.tm_year,
	    conf_info.creation_time.tm_mon+1, conf_info.creation_time.tm_mday);
    return_struct->create_time = malloc(6);
    sprintf(return_struct->create_time, "%02d:%02d", conf_info.creation_time.tm_hour, 
	    conf_info.creation_time.tm_min);
    
    /* Get the date and time for last written message */
    
    return_struct->last_written_date = malloc(9);
    sprintf(return_struct->last_written_date, "%02d-%02d-%02d", 
	    conf_info.last_written.tm_year, conf_info.last_written.tm_mon+1, 
	    conf_info.last_written.tm_mday);
    return_struct->last_written_time = malloc(6);
    sprintf(return_struct->last_written_time,"%02d:%02d",conf_info.last_written.tm_hour,
	    conf_info.last_written.tm_min);
    
    /* Get number of members in meeting */
    
    return_struct->num_members = conf_info.no_of_members;
    
    /* Get number of messages in meeting */
    
    return_struct->num_of_messages = conf_info.no_of_texts;
    
    /* Get the type of the meeting */
    
    if(conf_info.type.letter_box)
      return_struct->meeting_type = MEETING_TYPE_MAILBOX;
    else
      if(conf_info.type.rd_prot) 
	return_struct->meeting_type = MEETING_TYPE_CLOSED;
      else
	if(conf_info.type.original)
	  return_struct->meeting_type = MEETING_TYPE_NO_COMMENTS;
	else
	  if(conf_info.type.secret)
	    return_struct->meeting_type = MEETING_TYPE_SECRET;
	  else 
	    return_struct->meeting_type = MEETING_TYPE_NORMAL;
}


void lyx_status_meeting_free(lyx_status_meeting_return *return_struct)
{
    
    free(return_struct->name);
    free(return_struct->creator);
    free(return_struct->create_date);
    free(return_struct->create_time);
    free(return_struct->last_written_time);
    free(return_struct->last_written_time);

    memset(return_struct,sizeof(lyx_status_meeting_return),0);
}

