#include <stdio.h>
#include <stdlib.h>

#include <s-string.h>
#include <kom-types.h>
#include <kom-errno.h>
#include <services.h>

#include <lyxkom.h>
#include <lyx_protos.h>
#include <lyxerr.h>
#include <lyxkom_global.h>
#include <gui_protos.h>

/*
 * NAME lyx_status_user.c
 *
 * CREATED           93-05-04 by Ajo
 * LAST CHANGED:     93-05-18 by Janne
 *
 * DESCRIPTION
 *
 * Call this function with a ptr to an allocated lyx_status_user_return-structure
 * This function fills it with following info for a specified user:
 *   
 * - Name of user
 * - Number for user  
 * - Hostname 
 * - First login date and time
 * - Last login date and time
 * - Total time present (in seconds)
 * - Total number of written messages
 *
 * All space will be allocated by this function, so don't forget to call
 * lyx_status_user_free() when you don't need the information more.
 *
 */

void lyx_status_user(int user_number, lyx_status_user_return *returlista)
{
    Person user_info = EMPTY_PERSON;
    Conference conf_info = EMPTY_CONFERENCE;
    char tempstr[80];

    /* Call the low-level functions to get informatiotion to process */
    
    if (kom_get_person_stat((Pers_no)user_number, &user_info) == FAILURE)
      GUI_error("Serverfel (12:1)",SERVER_ERROR);
    if (kom_get_conf_stat((Conf_no)user_number, &conf_info) == FAILURE)
      GUI_error("Serverfel (12:2)",SERVER_ERROR);
    
    /* Get the name of the user */
    
    returlista->name = s_crea_c_str(conf_info.name);
    
    /* Put the argument 'user_number' in the struct */
    
    returlista->number = user_number;
    
    /* Get the date and time of user's first login *
       
     * By some strange, and to me unknown reason,  *
     * the 'month' is 1 less than it should be, so *
     * the '+1' is just a way to get it allright...*/
    
    returlista->first_login_date = malloc(9);
    sprintf(returlista->first_login_date, "%02d-%02d-%02d", conf_info.creation_time.tm_year, conf_info.creation_time.tm_mon+1, conf_info.creation_time.tm_mday);
    returlista->first_login_time = malloc(6);
    sprintf(returlista->first_login_time, "%02d:%02d", conf_info.creation_time.tm_hour, conf_info.creation_time.tm_min);
    
    
    /* Get the date of users last login */
    
    returlista->last_login_date = malloc(9);
    sprintf(returlista->last_login_date, "%02d-%02d-%02d", user_info.last_login.tm_year, user_info.last_login.tm_mon+1, user_info.last_login.tm_mday);
    returlista->last_login_time = malloc(6);	
    sprintf(returlista->last_login_time, "%02d:%02d", user_info.last_login.tm_hour, user_info.last_login.tm_min);
    
    /* Get the user's total time online */
    
    returlista->total_time = (int)user_info.total_time_present;
    
    /* Get the number of messages the user has written */
    
    returlista->num_written_messages = (int)user_info.no_of_created_texts;
    
    /* Get the user's hostname */
    
    returlista->host = s_crea_c_str(user_info.username);
}


void lyx_status_user_free(lyx_status_user_return *returlista)
{
    
    free(returlista->name);
    free(returlista->host);
    free(returlista->first_login_date);
    free(returlista->first_login_time);
    free(returlista->last_login_time);
    free(returlista->last_login_time);
    
    memset(returlista,sizeof(lyx_status_user_return),0);
}

