#include <stdio.h>
#include <stdlib.h>

#include <s-string.h>
#include <kom-types.h>
#include <kom-errno.h>
#include <services.h>

#include <lyxkom.h>
#include <lyx_protos.h>
#include <lyxerr.h>
#include <lyxkom_global.h>
#include <gui_protos.h>

/*
 * NAME lyx_who_is_on
 *
 * CREATED           93-04-28 by Ajo
 * LAST CHANGED:     93-05-18 by Janne
 *
 * DESCRIPTION
 *
 * Call this function with a ptr to an allocated lyx_who_is_on_return-structure.
 * This function fills it with following info for each user:
 *
 *   Users full name
 *   Users number
 *   Active meeting
 *   What the user is doing
 *
 * All space will be allocated by this function, so don't forget to call
 * lyx_who_is_on_free() when you don't need the information more.
 *
 */

void lyx_who_is_on(lyx_who_is_on_return *return_struct)
{
    int i, j;
    char *tmpstr;

    Who_info_list infolist = EMPTY_WHO_INFO_LIST;
    Conference conference = EMPTY_CONFERENCE;

    if (kom_who_is_on(&infolist) == FAILURE)
      GUI_error("Serverfel (15:1)",SERVER_ERROR);
    return_struct->no_of_users = infolist.no_of_persons;

    return_struct->number = malloc(infolist.no_of_persons*sizeof(int));
    return_struct->name = malloc(infolist.no_of_persons*sizeof(char *));
    return_struct->conference = malloc(infolist.no_of_persons*sizeof(char *));
    return_struct->doing = malloc(infolist.no_of_persons*sizeof(char *));
    
    if(return_struct->number == NULL || return_struct->name == NULL ||
       return_struct->conference == NULL || return_struct->doing == NULL)
      GUI_error("Minnet slut (15:2)",MEMORY_ERROR);
    
    for (i=0; i<infolist.no_of_persons; i++) {
        return_struct->number[i] = infolist.info[i].person;
	
	/* Get (and save)the name of the user */
	
        if (kom_get_conf_stat(infolist.info[i].person, &conference) == FAILURE)
	  GUI_error("Serverfel (15:3)",SERVER_ERROR);
	
        return_struct->name[i] = s_crea_c_str(conference.name);
	
        if(return_struct->name[i]==NULL)
            GUI_error("Minnet slut (15:4)",MEMORY_ERROR);

	/* Get what this user is doing */

        return_struct->doing[i] = s_crea_c_str(infolist.info[i].what_am_i_doing);
        if(return_struct->doing[i]==NULL)
	  GUI_error("Minnet slut (15:5)",MEMORY_ERROR);

	/* Get active conf */

        if(infolist.info[i].working_conference == 0) {
            return_struct->conference[i] = malloc(100);
            if(return_struct->conference[i]==NULL)
	      GUI_error("Minnet slut (15:6)",MEMORY_ERROR);
	    
            strcpy(return_struct->conference[i],"inget mte");
            continue;
        }
	
	/* This is really an "else"-statement, but I prefer this trick. */
	
        if (kom_get_conf_stat(infolist.info[i].working_conference, &conference)
	    == FAILURE)
	  GUI_error("Serverfel (15:7)", SERVER_ERROR);
        return_struct->conference[i] = s_crea_c_str(conference.name);
        if(return_struct->conference[i]==NULL)
	  GUI_error("Minnet slut (15:8)",MEMORY_ERROR);
    }
}

void lyx_who_is_on_free(lyx_who_is_on_return *return_struct)
{
    int i;
    
    for(i=0; i<return_struct->no_of_users; i++) {
        free(return_struct->doing[i]);
        free(return_struct->conference[i]);
        free(return_struct->name[i]);
    }
    free(return_struct->doing);
    free(return_struct->conference);
    free(return_struct->name);
    free(return_struct->number);
    memset(return_struct,sizeof(lyx_who_is_on_return),0);
}
